@@ -3,12 +3,12 @@
 %%% Author  : Alexey Shchepin <alexey@sevcom.net>
 %%% Purpose : 
 %%% Created : 23 Nov 2002 by Alexey Shchepin <alexey@sevcom.net>
-%%% Id      : $Id: jlib.erl 265 2004-09-17 19:52:59Z aleksey $
+%%% Id      : $Id: jlib.erl 412 2005-09-15 18:32:05Z alexey $
 %%%----------------------------------------------------------------------
 
 -module(jlib).
 -author('alexey@sevcom.net').
--vsn('$Revision$ ').
+-vsn('$Revision: 412 $ ').
 
 -export([make_result_iq_reply/1,
 	 make_error_reply/3,
@@ -219,8 +219,7 @@ jid_to_string({Node, Server, Resource}) ->
 is_nodename([]) ->
     false;
 is_nodename(J) ->
-    nodeprep(J).
-
+    nodeprep(J) /= error.
 
 
 %tolower_c(C) when C >= $A, C =< $Z ->
