@@ -3,16 +3,16 @@
 %%% Author  : Alexey Shchepin <alexey@sevcom.net>
 %%% Purpose : Roster management
 %%% Created : 11 Dec 2002 by Alexey Shchepin <alexey@sevcom.net>
-%%% Id      : $Id: mod_roster.erl 307 2005-04-17 18:08:34Z tmallard $
+%%% Id      : $Id: mod_roster.erl 459 2005-12-11 19:48:31Z alexey $
 %%%----------------------------------------------------------------------
 
 -module(mod_roster).
 -author('alexey@sevcom.net').
--vsn('$Revision$ ').
+-vsn('$Revision: 459 $ ').
 
 -behaviour(gen_mod).
 
--export([start/1, stop/0,
+-export([start/2, stop/1,
 	 process_iq/3,
 	 process_local_iq/3,
 	 get_user_roster/2,
@@ -28,41 +28,41 @@
 -include("mod_roster.hrl").
 
 
-start(Opts) ->
+start(Host, Opts) ->
     IQDisc = gen_mod:get_opt(iqdisc, Opts, one_queue),
     mnesia:create_table(roster,[{disc_copies, [node()]},
 				{attributes, record_info(fields, roster)}]),
     update_table(),
     mnesia:add_table_index(roster, us),
-    ejabberd_hooks:add(roster_get,
+    ejabberd_hooks:add(roster_get, Host,
 		       ?MODULE, get_user_roster, 50),
-    ejabberd_hooks:add(roster_in_subscription,
+    ejabberd_hooks:add(roster_in_subscription, Host,
 		       ?MODULE, in_subscription, 50),
-    ejabberd_hooks:add(roster_out_subscription,
+    ejabberd_hooks:add(roster_out_subscription, Host,
 		       ?MODULE, out_subscription, 50),
-    ejabberd_hooks:add(roster_get_subscription_lists,
+    ejabberd_hooks:add(roster_get_subscription_lists, Host,
 		       ?MODULE, get_subscription_lists, 50),
-    ejabberd_hooks:add(roster_get_jid_info,
+    ejabberd_hooks:add(roster_get_jid_info, Host,
 		       ?MODULE, get_jid_info, 50),
-    ejabberd_hooks:add(remove_user,
+    ejabberd_hooks:add(remove_user, Host,
 		       ?MODULE, remove_user, 50),
-    gen_iq_handler:add_iq_handler(ejabberd_sm, ?NS_ROSTER,
+    gen_iq_handler:add_iq_handler(ejabberd_sm, Host, ?NS_ROSTER,
 				  ?MODULE, process_iq, IQDisc).
 
-stop() ->
-    ejabberd_hooks:delete(roster_get,
+stop(Host) ->
+    ejabberd_hooks:delete(roster_get, Host,
 			  ?MODULE, get_user_roster, 50),
-    ejabberd_hooks:delete(roster_in_subscription,
+    ejabberd_hooks:delete(roster_in_subscription, Host,
 			  ?MODULE, in_subscription, 50),
-    ejabberd_hooks:delete(roster_out_subscription,
+    ejabberd_hooks:delete(roster_out_subscription, Host,
 			  ?MODULE, out_subscription, 50),
-    ejabberd_hooks:delete(roster_get_subscription_lists,
+    ejabberd_hooks:delete(roster_get_subscription_lists, Host,
 			  ?MODULE, get_subscription_lists, 50),
-    ejabberd_hooks:delete(roster_get_jid_info,
+    ejabberd_hooks:delete(roster_get_jid_info, Host,
 			  ?MODULE, get_jid_info, 50),
-    ejabberd_hooks:delete(remove_user,
+    ejabberd_hooks:delete(remove_user, Host,
 			  ?MODULE, remove_user, 50),
-    gen_iq_handler:remove_iq_handler(ejabberd_sm, ?NS_ROSTER).
+    gen_iq_handler:remove_iq_handler(ejabberd_sm, Host, ?NS_ROSTER).
 
 
 -define(PSI_ROSTER_WORKAROUND, true).
@@ -106,11 +106,11 @@ process_local_iq(From, To, #iq{type = Type} = IQ) ->
 
 
 
-process_iq_get(From, _To, #iq{sub_el = SubEl} = IQ) ->
+process_iq_get(From, To, #iq{sub_el = SubEl} = IQ) ->
     LUser = From#jid.luser,
     LServer = From#jid.lserver,
     US = {LUser, LServer},
-    case catch ejabberd_hooks:run_fold(roster_get, [], [US]) of
+    case catch ejabberd_hooks:run_fold(roster_get, To#jid.lserver, [], [US]) of
 	Items when is_list(Items) ->
 	    XItems = lists:map(fun item_to_xml/1, Items),
 	    IQ#iq{type = result,
@@ -313,11 +313,12 @@ push_item(User, Server, From, Item) ->
 			  push_item(User, Server, Resource, From, Item)
 		  end, ejabberd_sm:get_user_resources(User, Server)).
 
-% TODO: don't push to those who not load roster
+% TODO: don't push to those who didn't load roster
 -ifdef(PSI_ROSTER_WORKAROUND).
 
 push_item(User, Server, Resource, _From, Item) ->
     ResIQ = #iq{type = set, xmlns = ?NS_ROSTER,
+		id = "push",
 		sub_el = [{xmlelement, "query",
 			   [{"xmlns", ?NS_ROSTER}],
 			   [item_to_xml(Item)]}]},
@@ -330,6 +331,7 @@ push_item(User, Server, Resource, _From, Item) ->
 
 push_item(User, Server, Resource, From, Item) ->
     ResIQ = #iq{type = set, xmlns = ?NS_ROSTER,
+		id = "push",
 		sub_el = [{xmlelement, "query",
 			   [{"xmlns", ?NS_ROSTER}],
 			   [item_to_xml(Item)]}]},
@@ -451,10 +453,16 @@ process_subscription(Direction, User, Server, JID1, Type) ->
 
 %% in_state_change(Subscription, Pending, Type) -> NewState
 %% NewState = none | {NewSubscription, NewPending}
+-ifdef(ROSTER_GATEWAY_WORKAROUND).
+-define(NNSD, {to, none}).
+-define(NISD, {to, in}).
+-else.
+-define(NNSD, none).
+-define(NISD, none).
+-endif.
 
 in_state_change(none, none, subscribe)    -> {none, in};
-in_state_change(none, none, subscribed)   -> {to, none}; % Workaround for gateways
-%in_state_change(none, none, subscribed)   -> none;
+in_state_change(none, none, subscribed)   -> ?NNSD;
 in_state_change(none, none, unsubscribe)  -> none;
 in_state_change(none, none, unsubscribed) -> none;
 in_state_change(none, out,  subscribe)    -> {none, both};
@@ -462,8 +470,7 @@ in_state_change(none, out,  subscribed)   -> {to, none};
 in_state_change(none, out,  unsubscribe)  -> none;
 in_state_change(none, out,  unsubscribed) -> {none, none};
 in_state_change(none, in,   subscribe)    -> none;
-in_state_change(none, in,   subscribed)   -> {to, in}; % Workaround for gateways
-%in_state_change(none, in,   subscribed)   -> none;
+in_state_change(none, in,   subscribed)   -> ?NISD;
 in_state_change(none, in,   unsubscribe)  -> {none, none};
 in_state_change(none, in,   unsubscribed) -> none;
 in_state_change(none, both, subscribe)    -> none;
