@@ -36,12 +36,12 @@ CREATE TABLE rostergroups (
 
 CREATE INDEX pk_rosterg_user_jid ON rostergroups USING btree (username, jid);
 
-;; To update from previous table definition:
-; CREATE SEQUENCE spool_seq_seq;
-; ALTER TABLE spool ADD COLUMN seq integer;
-; ALTER TABLE spool ALTER COLUMN seq SET DEFAULT nextval('spool_seq_seq');
-; UPDATE spool SET seq = DEFAULT;
-; ALTER TABLE spool ALTER COLUMN seq SET NOT NULL;
+--- To update from previous table definition:
+-- CREATE SEQUENCE spool_seq_seq;
+-- ALTER TABLE spool ADD COLUMN seq integer;
+-- ALTER TABLE spool ALTER COLUMN seq SET DEFAULT nextval('spool_seq_seq');
+-- UPDATE spool SET seq = DEFAULT;
+-- ALTER TABLE spool ALTER COLUMN seq SET NOT NULL;
 
 CREATE TABLE spool (
     username text NOT NULL,
